local addonName = ...
local Master = LibStub('AceAddon-3.0'):GetAddon(GetAddOnDependencies(addonName))
local Addon = Master:CreateClass('Frame', Master.Frame)
Addon.name = addonName
_G[addonName] = Addon

Addon.master = Master
Addon.lib = LibStub('LibSharedMedia-3.0', true)

Addon.frames = {}

function Addon:New(name)
	local name = string.lower(name)
	if name == "vehicle" then
		name = "VEHICLE"
	end
	local f = self.proto.New(self, name)
	f.container = f:CreateTexture(nil, 'ARTWORK')
	f.container:SetPoint("Center", f)
	f:Reload()
	--f:SetParent(self.header)
	Addon.frames[name] = Addon.frames[name] or f
	return f
end


function Addon:GetDefaults()
	local def
	if self and self.id and self.defaults[self.id] then 
		def = Addon.Copy(Addon.defaults[self.id])
	else
		def = Addon.Copy(Addon.defaults["focus"])
		def.x = 0
		def.y = 0
		def.point = "Center"
	end
	return def
end

function Addon:Reload()
	if self.Load then
		self:Load()
	end
	self:SetParent(self.parent or self:GetParent())
	self:Layout()
end

function Addon:Unload()
	local p = Addon.Copy(self:GetDefaults(), Addon.master.db.profile.frames[self.id])
	Addon.master.db.profile.frames[self.id] = p
	self.sets = p
	self:Layout()
	self:Rescale()
	self:Reposition()
	self:Reanchor()

end

function Addon:Disable()
	self:Free()
end


function Addon:Delete()
	Addon.master.db.profile.frames[self.id] = false
	self:Free()
end

function Addon:Layout()
	if self.UpdateWidgets then
		self:UpdateWidgets()
	else
		local pad = self:GetPadding()
		local w, h = self.sets.width, self.sets.height
		self:SetSize(w + pad, h + pad)
		self.container:SetPoint("Center", self)
		self.container:SetSize(w,h)
		self.sets.frameStrata = self.sets.frameStrata or 1
		self.sets.frameLayer = self.sets.frameLayer or 1
		local lay = Addon.layers[self.sets.frameStrata]
		self:SetFrameStrata(lay)
		self:SetFrameLevel(self.sets.frameLayer)
	end
end


local mod = Addon.master:NewModule('Frame')

local enabledFrames = {}

function mod:OnInitialize()
	Addon.master.db.profile.units = Addon.master.db.profile.units or defaults

end

function mod:Load()
	wipe(enabledFrames)
	for unit, b in pairs(Addon.getSets().frames) do
		if b == true then
			unit = string.lower(unit)
			local frame = Addon:New(unit)
			tinsert(enabledFrames, frame)
		end
	end	

	for frameName, b in pairs(Addon.getSets().hide) do
		if frameName and _G[frameName] then
			_G[frameName]:SetParent(Addon.HIDER)
		end
	end
end

function mod:Unload()
	if enabledFrames then
		for i, frame in pairs(enabledFrames) do
			Addon.master.Frame:ForFrame(frame.id, 'Free')
			frame:Unload()
		end
	end
end
