local Addon = _G[...]

do --simple way to hide UI elements
	Addon.HIDER = CreateFrame("Frame")
	Addon.HIDER:Hide()
end

do --Prevents a frame being made for a unit that can't exist.

	local baseUnits = {
		'player',
		'target',
		'playerpet',
		'focus',
		'arena%d+',
		'boss%d+',
		'party%d+',
		'partypet%d+',
		'raid%d+',
		'raidpet%d+',
		'mouseover',
	}

	local maxUnits = {
		raid = 40,
		raidpet = 40,
		arena = 4,
		boss = 4,
		party = 4,
		partypet = 4,
	}

	local function IsUnitValid(unit, u)
		local keep = u or unit
		unit = string.lower(u or unit)
		local count = 0
		if string.match (unit, 'target' , 2) then
			_, count = string.gsub(string.sub (unit, 2, string.len(unit)), 'target', '')
			if ((unit == 'player') and count > 0) or (count > 5) then
				return nil
			else
				unit = string.gsub (unit, 'target', '', count)
			end
		end
		for i, unitString in pairs(baseUnits) do
			if string.match(unit, unitString) then
				local base = string.gsub(unit, '%d+', '')
				local num = string.gsub(unit, base, '')
				local extra = string.gsub(unit, num, '')
				if extra == base then
					local Max = maxUnits[base]
					if (not Max) or (Max and (Max >= tonumber(num))) then 
						return string.lower(keep)
					end
				end
			end
		end
	end

	Addon.IsUnitValid = IsUnitValid

end

do --slash Commands
	local frames = Addon.frames

	local commands = Addon.master:GetModule('SlashCommands')

	local oldOnCmd = commands.OnCmd

	local cmds = {
		new = function(unit)
			unit = string.lower(unit)
			if unit == "vehicle" then
				unit = "VEHICLE"
			end
			if not Addon:IsUnitValid(unit) then
				return print('invalid unit! :(', unit)
			end
			Addon.frames[unit] = Addon.frames[unit] or Addon:New(unit)
			if Addon.frames[unit].frameOverlay then
				Addon.frames[unit].frameOverlay:Show()
			end
			Addon.frames[unit]:Reload()
			Addon.frames[unit]:Restore()
			Addon.frames[unit]:Show()
			
			Addon.frames[unit]:Layout()
			Addon.frames[unit]:EnableMouse(true)
			if LibStub and LibStub('LibKeyBound-1.0') then
				LibStub('LibKeyBound-1.0'):Deactivate()
			end
							Addon.master:SetLock(true)
				Addon.master:SetLock(nil)
			Addon.master.db.profile.units.frames[unit] = true
		end,
		delete = function(unit)
			unit = string.lower(unit)
			if frames[unit] then
				frames[unit]:Delete()
				Addon.master:GetModule('ConfigOverlay'):Hide()
			end
				Addon.master:SetLock(true)
				Addon.master:SetLock(nil)
		end,
		show = function (arg)
			if _G[arg] then
				Addon.getSets().hide[arg] = nil
				_G[arg]:SetParent(_G[arg].par_ent)
			end
		end,
		hide = function (arg)
			if _G[arg] and not (Addon.getSets().hide[arg]) then
				Addon.getSets().hide[arg] = true
				_G[arg].par_ent = _G[arg].par_ent or _G[arg]:GetParent()
				_G[arg]:SetParent(Addon.HIDER)
				print(arg, "hidden")
			end
		end,
		copy = function(copy)
			local enabled = Addon:getSets()
			local files = Addon.Profiles.GetProfiles()
			local toCopy = files[copy]

			enabled.frames = {}
			local active = Addon.Profiles:GetActiveProfile()
			for unit, sets in pairs(toCopy) do
		--		print(unit, active.units.frames[unit])
				active.units.frames[unit] = true
				active.frames[unit] = Addon.Copy(sets)
				local frame = Addon.frames[unit] or Addon:New(unit)
				frame.sets = active.frames[unit]
				Addon.master:SetLock(nil)
				Addon.master:SetLock(true)
				if frame.frameOverlay then
					frame.frameOverlay:Show()
				end
				frame:Reload()
				frame:Restore()
				frame:Layout()
				frame:Show()
				frame:EnableMouse(true)
				if LibStub and LibStub('LibKeyBound-1.0') then
					LibStub('LibKeyBound-1.0'):Deactivate()
				end
				Addon.master:GetModule('ConfigOverlay'):Show()
			end
		end
		}

	function commands:OnCmd(message)
		local arg1, arg2, arg3  = string.split(' ', message)
		if arg1:lower() == 'unit' then
			if cmds[arg2:lower()] then
				cmds[arg2:lower()](arg3)
			end
			return
		end
		oldOnCmd(self, message)
	end
end

do --table manipulation
	local function Copy(source, destination, looped, bolackList)
		if not destination then
			destination = {}
		end
		for i, b in pairs((source)) do
			if (type(b) == 'table') then
				destination[i] = Copy(b, destination[i], true, blackList)
			else
				if (not looped) and (blackList and blackList[i]) then
					
				else
					destination[i] = b
				end
			end
		end
		return destination
	end
	Addon.Copy = Copy
	
	local function Blend(source, destination)
		destination = destination or {}
		if not source then
			return destination
		end

		for key, value in pairs(CopyTable(source)) do
			if (type(value) == 'table') then
				if type(key) == "number" then
					local index = #destination + 1
					destination[index] = Blend(value, destination[index])
				else
					destination[key] = Blend(value, destination[key])
				end
			else
				destination[key] = destination[key] or value
			end
		end
		return destination
	end

	Addon.Blend = Blend

	local function Merge(source, destination)
		destination = destination or {}
		if not source then
			return destination
		end

		for key, value in pairs(CopyTable(source)) do
			if (type(value) == 'table') then
				if type(key) == "number" then
					local index = #destination + 1
					destination[index] = Merge(value, destination[index])
				else
					destination[key] = Merge(value, destination[key])
				end
			else
				destination[key] = value
			end
		end
		return destination
	end

	Addon.Merge = Merge
	
	local function check(source, target)
		--you may now add new defaults at will. ~Goranaws
		if not target then 
			target = {}
		end
		if not source then
			--return
		end
		for key, value in pairs(source) do
			if type(value) == 'table' then
				target[key] = check(value, target[key])
			else
				if (type(value) == 'boolean') then
					if target[key] == nil then
						target[key] = value
					end
				else
					target[key] = target[key] or value
				end
			end
		end
		return target
	end
	Addon.check = check
	
	
end

do -- returns a function that generates unique names for frames
	-- in the format <AddonName>_<Prefix>[1, 2, ...]
	local generators = {

	}
	function Addon:CreateNameGenerator(prefix)
		if generators[prefix] then
			return generators[prefix]
		end

		local id = 0
		local func =  function()
			id = id + 1
			return ('%s_%s_%d'):format('DominosUnitsOptions', prefix, id)
		end
		generators[prefix] = func
		return func
	end
end

do -- extra textures
	local lib = Addon.lib

	Addon.statusbarTextures = {
		ArcHUD = [[Interface\AddOns\Dominos_Units\textures\ArcHUD.tga]],
		ArcHUD2 = [[Interface\AddOns\Dominos_Units\textures\ArcHUD2.tga]],
		Bar = [[Interface\AddOns\Dominos_Units\textures\Bar.tga]],
		Bar2 = [[Interface\AddOns\Dominos_Units\textures\Bar2.tga]],
		BloodGlaives = [[Interface\AddOns\Dominos_Units\textures\BloodGlaives.tga]],
		BloodGlaives2 = [[Interface\AddOns\Dominos_Units\textures\BloodGlaives2.tga]],
		CleanCurves = [[Interface\AddOns\Dominos_Units\textures\CleanCurves.tga]],
		CleanCurves2 = [[Interface\AddOns\Dominos_Units\textures\CleanCurves2.tga]],
		CleanCurvesOut = [[Interface\AddOns\Dominos_Units\textures\CleanCurvesOut.tga]],
		CleanCurvesOut2 = [[Interface\AddOns\Dominos_Units\textures\CleanCurvesOut2.tga]],
		CleanTank = [[Interface\AddOns\Dominos_Units\textures\CleanTank.tga]],
		CleanTank2 = [[Interface\AddOns\Dominos_Units\textures\CleanTank2.tga]],
		ColorBar = [[Interface\AddOns\Dominos_Units\textures\ColorBar.tga]],
		ColorBar2 = [[Interface\AddOns\Dominos_Units\textures\ColorBar2.tga]],
		ComboCleanCurves = [[Interface\AddOns\Dominos_Units\textures\ComboCleanCurves.blp]],
		DHUD = [[Interface\AddOns\Dominos_Units\textures\DHUD.tga]],
		DHUD2 = [[Interface\AddOns\Dominos_Units\textures\DHUD2.tga]],
		FangRune = [[Interface\AddOns\Dominos_Units\textures\FangRune.tga]],
		FangRune2 = [[Interface\AddOns\Dominos_Units\textures\FangRune2.tga]],
		GemTank = [[Interface\AddOns\Dominos_Units\textures\GemTank.tga]],
		GlowArc = [[Interface\AddOns\Dominos_Units\textures\GlowArc.tga]],
		GlowArc2 = [[Interface\AddOns\Dominos_Units\textures\GlowArc2.tga]],
		HiBar = [[Interface\AddOns\Dominos_Units\textures\HiBar.tga]],
		HiBar2 = [[Interface\AddOns\Dominos_Units\textures\HiBar2.tga]],
		PillTank = [[Interface\AddOns\Dominos_Units\textures\PillTank.tga]],
		RivetBar = [[Interface\AddOns\Dominos_Units\textures\RivetBar.tga]],
		RivetBar2 = [[Interface\AddOns\Dominos_Units\textures\RivetBar2.tga]],
		RoundBar = [[Interface\AddOns\Dominos_Units\textures\RoundBar.tga]],
		RoundBar2 = [[Interface\AddOns\Dominos_Units\textures\RoundBar2.tga]],
		RuneBar = [[Interface\AddOns\Dominos_Units\textures\RuneBar.tga]],
		RuneBar2 = [[Interface\AddOns\Dominos_Units\textures\RuneBar2.tga]],
	}

	for textureName, texturePath in pairs(Addon.statusbarTextures) do
		if not lib.MediaTable.statusbar[textureName] then
			lib.MediaTable.statusbar[textureName] = texturePath
		end
	end
end