local Addon = _G[...]
local modName = ...

--Unit Menu
local widget = Addon:NewWidget('drop', 'Button')

function widget:New(parent)
	local name = ('%s_%s_UnitMenu'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local button = self:Bind(CreateFrame('Frame', name, parent.box, 'L_UIDropDownMenuTemplate'))
	if button then
		button.owner = parent
	end
	self.unitButton = button
	return button
end

function widget:GetDefaults()
    return {}
end

local shortcut = {
	party = "PARTY",
	boss = "BOSS",
	focus = "FOCUS",
	arenapet = "ARENAENEMY",
	arena = "ARENAENEMY",
	player = "SELF",
	vehicle = "VEHICLE",
	pet = "PET"
}

function widget:Initialize()
	local unit = self.id
	if not unit then return end
	
	local unitType = string.match(unit, "^([a-z]+)[0-9]+$") or unit;

	-- Mimic the default UI and prefer the relevant units menu when possible
	local menu = shortcut[unitType]
	
	if not menu then
		-- Then try and detect the unit type and show the most relevant menu we can find
		if( UnitIsUnit(unit, "player") ) then
			menu = shortcut["player"]
		elseif( UnitIsUnit(unit, "vehicle") ) then
			menu = shortcut["vehicle"]
		elseif( UnitIsUnit(unit, "pet") ) then
			menu = shortcut["pet"]
		elseif( UnitIsOtherPlayersBattlePet(unit) ) then
			menu = "OTHERBATTLEPET";
		elseif( UnitIsOtherPlayersPet(unit) ) then
			menu = "OTHERPET";
		-- Last ditch checks 
		elseif( UnitIsPlayer(unit) ) then
			if( UnitInRaid(unit) ) then
				menu = "RAID_PLAYER";
			elseif( UnitInParty(unit) ) then
				menu = shortcut[party]
			else
				menu = "PLAYER";
			end
		elseif( UnitIsUnit(unit, "target") ) then
			menu = "TARGET";
		end
	end
	if( menu ) then
		L_UnitPopup_ShowMenu(self, menu, unit);
	end
end

local function SetMenuFunc(frame, menuFunc)	
	L_UIDropDownMenu_Initialize(frame.dropDown, menuFunc, "MENU");
	frame.menu = function()
		L_ToggleDropDownMenu(1, 1, frame.dropDown, frame:GetName(), 0, 0, nil, nil, 5)
	end
end

function widget:Load()
	self.id = self.owner.id
	self.owner.box.dropDown = self
	self.owner.box.id = self.owner.id
	self.owner.box:SetAttribute('*type1', "target")
	self.owner.box:SetAttribute('*type2', 'menu')
	self.owner.box:RegisterForClicks('LeftButtonUp', 'RightButtonUp')
	self:SetFrameStrata("LOW")
	if self.Initialize then
		SetMenuFunc(self.owner.box, self.Initialize)
	end
end
