local Addon = _G[...]
local modName = ...

--Level Text
local widget = Addon:NewWidget('level', 'Frame')
widget.Events = {
	["UNIT_LEVEL"] = 1,
}

function widget:New(parent)
	local name = ('%s_%s_LevelText'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local control = self:Bind(CreateFrame('Frame', name, parent.box))
	control.owner = parent
	control:SetFrameLevel(40)
	local string = control:CreateFontString(nil, 'ARTWORK', 'GameFontNormalSmall')
	string:SetAllPoints(control)
	control.text = string
	return control
end

widget.defaults = {
	scale = 100,
	anchor = 'Center',
	x = 63,
	y = -16,
	justifyH = 2,		
	enable = true,
	font = "Friz Quadrata TT",
	size = 9,
	color = {r = 1,b = 0, g = 1, a = 1},
	frameLevel = 5,
	frameStrata = 2,
}

local _hori = {
	'LEFT',
	'CENTER',
	'RIGHT',
}

local _vert = {
	'TOP',
	'MIDDLE',
	'BOTTOM',
}

function widget:Layout()
	local sets = self.sets
	if sets.enable ~= true then
		self:Hide()
		self.text:SetText('')
		return
	else
		self:Show()
	end
	self.text:SetJustifyH(_hori[sets.justifyH]) -- Sets horizontal text justification ('LEFT','RIGHT', or 'CENTER')
	self:ClearAllPoints()
	local x = self.sets.x/(self.sets.scale/100)
	local y = self.sets.y/(self.sets.scale/100)
	self:ClearAllPoints()
	self.text:ClearAllPoints()
	self:SetAllPoints(self.text)
	self.text:SetPoint("Center", self:GetParent(), self.sets.anchor, x, y)
	
	local path = self:GetMediaPath(self.sets.font)
	self.text:SetFont(path, self.sets.size or 12)
	local c = self.sets.color
	self.text:SetTextColor(c.r,c.g,c.b,c.a)
end

function widget:GetMediaPath(fileName)
	if Addon.lib then
		self.sets.texturePath = Addon.lib and Addon.lib:Fetch("font", fileName)
	end
	return (Addon.lib and Addon.lib:Fetch("font", fileName))
end

function widget:Update()
	if self.sets.enable ~= true then
		return
	end
	local unit = self.owner.id
	local lvl = UnitLevel(unit)
	if lvl <= 0 then
		lvl = "??"
	else
		lvl = math.abs(lvl)
	end
	self.text:SetText(lvl)
end

widget.Options = {
                    {
                        name = 'Justify Hori',
                        kind = 'Slider',
                        key = 'justifyH',
                        min = 1,
                        max = 3,
						panel = "advanced",
                    },
					{
						name = 'Font',
						kind = 'Media',
						key = 'font',
						mediaType = 'Font',
						panel = "basic",
					},
                    {
                        name = 'Size',
                        kind = 'Slider',
                        key = 'size',
                        min = 1,
                        max = 25,
						panel = "basic",
                    },
					{
                        name = 'Color',
                        kind = 'ColorPicker',
                        key = 'color',
						panel = "basic",
                    },
        }