local Addon = _G[...]
local modName = ...

local widget = Addon:NewWidget('overlay', 'Frame')
widget.Events = {
	UNIT_CLASSIFICATION_CHANGED = 1,
}

function widget:New(parent)
	local name = ('%s_%s_overlay'):format(modName, parent.id)
	if _G[name] then
		return _G[name]
	end
	local bar = self:Bind(CreateFrame('Frame', name, parent.box))
	bar:EnableMouse(false)

	bar.texture = bar:CreateTexture(nil, 'ARTWORK')
	bar.texture:SetAllPoints(bar)

	bar.left = bar:CreateTexture(nil, 'ARTWORK')
	bar.right = bar:CreateTexture(nil, 'ARTWORK')
	bar.middle = bar:CreateTexture(nil, 'ARTWORK')

	bar.owner = parent

	bar.leftGlow = bar:CreateTexture(nil, 'ARTWORK')
	bar.leftGlow:SetTexture('Interface\\CharacterFrame\\UI-Player-Status')
	bar.leftGlow:SetTexCoord(0, 0.74609375, 0, 0.53125)
	bar.leftGlow:SetBlendMode('ADD')	
	
	bar.middleGlow = bar:CreateTexture(nil, 'ARTWORK')
	bar.middleGlow:SetTexture('Interface\\CharacterFrame\\UI-Player-Status')
	bar.middleGlow:SetTexCoord(0, 0.74609375, 0, 0.53125)
	bar.middleGlow:SetBlendMode('ADD')
	
	bar.rightGlow = bar:CreateTexture(nil, 'ARTWORK')
	bar.rightGlow:SetTexture('Interface\\CharacterFrame\\UI-Player-Status')
	bar.rightGlow:SetTexCoord(0, 0.74609375, 0, 0.53125)
	bar.rightGlow:SetBlendMode('ADD')

	bar.rightGlow:Hide()
	bar.leftGlow:Hide()
	bar.middleGlow:Hide()

	parent.threatIndicator = bar.leftGlow
	parent.unit = parent.id
	bar.leftGlow.unit = parent.id
	bar.statusCounter = 0
	bar.statusSign = -1
	bar:SetAllPoints(self)
	return bar
end

widget.defaults = {
	enable = true,
	anchor = 'Center',
	x = 0,
	y = 0,
	flipL2R = false,
	frameLevel = 4,
	frameStrata = 2,
}

function widget:Load()
	self:EnableMouse(false)
	self.noMouse = true
	self.id = self.owner.id
end

function widget:Layout()
	local sets = self.sets
	if sets.enable ~=true then
		self:Hide()
		self.noUpdate = true
		return
	else
		self:Show()
		self.noUpdate = nil
	end

	self:EnableMouse(false)
	self:ClearAllPoints()
	self:SetAllPoints(self:GetParent())
	self.left:ClearAllPoints()
	self.right:ClearAllPoints()
	self.middle:ClearAllPoints()
	self.leftGlow:ClearAllPoints()
	self.rightGlow:ClearAllPoints()
	self.middleGlow:ClearAllPoints()
	
	if self.sets.flipL2R == true then
		self.left:SetPoint("TopLeft", 0, 0)
		self.left:SetPoint("BottomRight", self, "BottomLeft", 115, 0)
		self.right:SetPoint("TopRight", 0, 0)
		self.right:SetPoint("BottomLeft", self, "BottomRight", -12, 0)
		self.middle:SetPoint("TopLeft", self.left, "TopRight", 0, 0)
		self.middle:SetPoint("BottomRight", self.right, "BottomLeft", 0, 0)
		self.leftGlow:SetPoint("TopLeft", 30, -8)
		self.leftGlow:SetPoint("BottomRight", self, "BottomLeft", 125, 25)
		self.rightGlow:SetPoint("TopRight", -7, -8)
		self.rightGlow:SetPoint("BottomLeft", self, "BottomRight", -15, 25)
		self.middleGlow:SetPoint("TopLeft", self.leftGlow, "TopRight", 0, 0)
		self.middleGlow:SetPoint("BottomRight", self.rightGlow, "BottomLeft", 0, 0)
	else
		self.left:SetPoint("TopRight", 0, 0)
		self.left:SetPoint("BottomLeft", self, "BottomRight", -115, 0)
		self.right:SetPoint("TopLeft", 0, 0)
		self.right:SetPoint("BottomRight", self, "BottomLeft", 12, 0)
		self.middle:SetPoint("TopRight", self.left, "TopLeft", 0, 0)
		self.middle:SetPoint("BottomLeft", self.right, "BottomRight", 0, 0)
		self.leftGlow:SetPoint("TopRight", -30, -8)
		self.leftGlow:SetPoint("BottomLeft", self, "BottomRight", -125, 25)
		self.rightGlow:SetPoint("TopLeft", 7, -8)
		self.rightGlow:SetPoint("BottomRight", self, "BottomLeft", 15, 25)
		self.middleGlow:SetPoint("TopRight", self.leftGlow, "TopLeft", 0, 0)
		self.middleGlow:SetPoint("BottomLeft", self.rightGlow, "BottomRight", 0, 0)
	end
end

function widget:Update(elapsed)
	if (self.sets.enable ~= true) or self.noUpdate then
		return
	end
    if ( UnitHasVehiclePlayerFrameUI(self.owner.id) ) then
		self.rightGlow:Hide()
		self.leftGlow:Hide()
		self.middleGlow:Hide()
	elseif (UnitIsUnit(self.owner.id, 'player')) and( IsResting() ) then
		self.rightGlow:SetVertexColor(1.0, 0.88, 0.25, 1.0)
		self.leftGlow:SetVertexColor(1.0, 0.88, 0.25, 1.0)
		self.middleGlow:SetVertexColor(1.0, 0.88, 0.25, 1.0)
		self.rightGlow:Show()
		self.leftGlow:Show()
		self.middleGlow:Show()
    else
		self.rightGlow:SetVertexColor(1,1,1,1)
		self.leftGlow:SetVertexColor(1,1,1,1)
		self.middleGlow:SetVertexColor(1,1,1,1)
		self.rightGlow:Hide()
		self.leftGlow:Hide()
		self.middleGlow:Hide()
    end
	if ( self.rightGlow:IsShown() ) then
		self.statusCounter = self.statusCounter or 0
		local alpha = 255
		local counter = self.statusCounter + (elapsed or 0)
		local sign    = self.statusSign
		if ( counter > 0.5 ) then
		    sign = -sign
		    self.statusSign = sign
		end
		counter = mod(counter, 0.5)
		self.statusCounter = counter
 
		if ( sign == 1 ) then
		    alpha = (55  + (counter * 400)) / 255
		else
		    alpha = (255 - (counter * 400)) / 255
		end
		self.rightGlow:SetAlpha(alpha)
		self.leftGlow:SetAlpha(alpha)
		self.middleGlow:SetAlpha(alpha)
    end

	local classification = UnitClassification(self.owner.id)
	if self.last_classification ~= classification then
	    self.last_classification = classification
	
        local texture
        if ( classification == 'minus' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame')
        elseif ( classification == 'worldboss' or classification == 'elite' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Elite')
        elseif ( classification == 'rareelite' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Rare-Elite')
        elseif ( classification == 'rare' ) then
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame-Rare')
        else
            texture = ('Interface\\TargetingFrame\\UI-TargetingFrame')
        end
		self.left:SetTexture(texture)
		self.right:SetTexture(texture)
        self.middle:SetTexture(texture)
    end
		local a, b, c, d = 1, .54, .13, .09375
		local aa, bb ,cc, dd = 0, .35, .7, 0.74609375		

		if self.sets.flipL2R ~= true then
			self.left:SetTexCoord(b, a,0,0.78125)
			self.right:SetTexCoord(d, c,0,0.78125)
			self.middle:SetTexCoord(b, c,0,0.78125)
			
			self.leftGlow:SetTexCoord(bb, aa, 0, 0.53125)
			self.middleGlow:SetTexCoord(cc ,bb, 0, 0.53125)
			self.rightGlow:SetTexCoord(dd, cc, 0, 0.53125)
		else
			self.left:SetTexCoord(a, b,0,0.78125)
			self.right:SetTexCoord(c, d,0,0.78125)
			self.middle:SetTexCoord(c, b,0,0.78125)
			
			self.leftGlow:SetTexCoord(aa, bb, 0, 0.53125)
			self.middleGlow:SetTexCoord(bb ,cc, 0, 0.53125)
			self.rightGlow:SetTexCoord(cc, dd, 0, 0.53125)
		end
	if self.sets.enable == true then
		self:SetAlpha(1)
	else
		self:SetAlpha(0)
	end
end

widget.Options = {
	
                    {
                        name = 'Flip',
                        kind = 'CheckButton',
                        key = 'flipL2R',
						panel = "basic",
                    },
}
